# -*- encoding:utf-8 -*-

# 経理システム (仮)
# Copyright (C) 2007 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/bside/


require 'ostruct'  # DEBUG
require 'digest/sha1'


# 売上側の請求書 (売掛金管理)
class SalesInvoicesController < ApplicationController
  before_filter :login_required
  append_after_filter :make_pdf, :only => [:print_invoice, :print_invoice_en]
  
  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ],
         :redirect_to => { :action => :list }

  def index
    list
    render :action => 'list'
  end
  
  # アトミックに一時ファイルを作ってファイル名を返す
  def new_filename dir
    hash = Digest::SHA1.new
    begin
      now = Time::now
      hash.update(now.to_s)
      hash.update(String(now.usec))
      hash.update(String(rand(0)))
      fn = hash.hexdigest
      # 大丈夫だと思うが、実際に作ってみる
      begin
        fp = File.open(dir + '/' + fn, File::WRONLY | File::CREAT | File::EXCL)
      rescue SystemCallError
      end
    end while !fp
    fp.close
    fn
  end
  private :new_filename

  # XSL-FOをPDFに変換する
  def make_pdf
    tmpdir = RAILS_ROOT + '/tmp'
    infn = tmpdir + '/' + new_filename(tmpdir)
    outfn = tmpdir + '/' + new_filename(tmpdir)
    errfn = tmpdir + '/' + new_filename(tmpdir)

    File.open(infn, 'w') {|f| f.write response.body }
    headers['Content-Type'] = 'application/pdf'
    `#{ENV['FOP_HOME']}/fop -c #{RAILS_ROOT}/config/fop-conf.xml #{infn} -pdf #{outfn} 2>#{errfn}`
    response.body = File.read(outfn)
    File.unlink infn
    File.unlink outfn
  end
  private :make_pdf
  
  # 作った請求書を出力
  def print_invoice
    @company = OpenStruct.new(
              :name => '株式会社 テスト商事',
              :url => 'www.yahoo!!!.com',
              :address => '福岡市博多区2丁目2-1 FooBarビル',
              :tel => '092-555-1111',
              :zip => '812-xxxx',
              :finance_dept => '財務経理部長　田中　太郎',
              :person => 'ほげ山ほげ太'
            )
    @invoice = SalesInvoice.find params[:id]
    @today = Date.today
    render :layout => false
  end
  
  def print_invoice_en
    @company = OpenStruct.new(
              :name_en => 'TEST ENGINEERING, Inc.',
              :url => 'www.yahoo!!!.com',
              :address_en => 'FooBar Bldg., 2-2-1, Hakata-ku, Fukuoka-city',
              :tel_en => '+81-92-555-1111',
              :zip => '812-xxxx',
              :finance_dept => '財務経理部長　田中　太郎',
              :person => 'ほげ山ほげ太'
            )
    @invoice = SalesInvoice.find params[:id]
    @today = Date.today
    render :layout => false
  end
  
  def list
    @sales_invoices = SalesInvoice.paginate :page => params[:page],
                                            :per_page => 20
  end

  def show
    @invoice = SalesInvoice.find(params[:id])
  end
  
  # 新規作成画面
  def new
    @partner = Partner.find params[:partner]
    @invoice = SalesInvoice.new
    @invoice.contact_name1 = @partner.name
  end
  
  # 明細行のレコードを作る
  def create_details new_exp
    result = []
    (0..8).each {|i| # magic
      pa = params["detail#{i}"]
      detail = SiDetail.new pa
      detail.attributes = {
        :sales_invoice => @invoice,
        :unit_price => (pa[:unit_price].blank? ? nil :
                          (pa[:unit_price].to_f * (10 ** new_exp) + 0.5).floor),
        :line_no => i + 1
      }
      if detail.unit_price.to_i != 0 && detail.qty.to_i != 0
        detail.amount = detail.unit_price * detail.qty
      else
        detail.amount = (pa[:amount].to_f * (10 ** new_exp) + 0.5).floor
      end
      result << detail
    }
    # 末尾の空白行を取り除く
    while last = result.last
      if last.amount != 0 || last.description.strip != ''
        break
      else
        result.pop
      end
    end
    return result
  end
  private :create_details

  # 作成
  def create
    @partner = Partner.find params[:partner]
    @invoice = SalesInvoice.new params[:invoice]
    @invoice.attributes = {
      :partner => @partner,
      :division => current_user.default_division,
      :created_by => current_user,
      :state => 0,
    }
    details = create_details(@invoice.currency.exponent)
    banks = OurBank.find params[:ba]

    SalesInvoice.transaction {
      details.each {|r|
        @invoice.si_details << r
      }

      if @invoice.save
        details.each {|r| r.save! }
        banks.each {|r|
          b = SiBank.new :sales_invoice => @invoice,
                         :our_bank => r
          b.save!
        }
        
        flash[:notice] = 'SalesInvoice was successfully created.'
        redirect_to :action => 'show', :id => @invoice
      else
        render :action => 'new', :partner => @partner
      end
    } # of transaction
  end
  
  # 編集画面
  def edit
    @invoice = SalesInvoice.find(params[:id])
    @partner = @invoice.partner
  end
  
  # 更新する
  def update
    orig_invoice = SalesInvoice.find params[:id]
    @partner = orig_invoice.partner
    @invoice = SalesInvoice.new params[:invoice] # 検証用のダミー
    details = create_details(@invoice.currency.exponent)
    details.each {|r| @invoice.si_details << r }
    
    if !@invoice.valid?
      @invoice.id = orig_invoice.id
      render :action => 'edit', :id => @invoice
      return
    end
    
    # 更新する
    SalesInvoice.transaction {
      @invoice = orig_invoice
      @invoice.attributes = params[:invoice]
      @invoice.save!

      # 詳細行は作り直す
      SiDetail.delete_all ['sales_invoice_id = ?', @invoice.id]
      details.each {|r|
        r.sales_invoice = @invoice
        r.save!
      }
      
      SiBank.delete_all ['sales_invoice_id = ?', @invoice.id]
      if params[:ba]
        banks = OurBank.find params[:ba]
        banks.each {|r|
          b = SiBank.new :sales_invoice => @invoice,
                       :our_bank => r
          b.save!
        }
      end
    } # of transaction

    flash[:notice] = '請求書は正常に更新されました。'
    redirect_to :action => 'show', :id => @invoice
  end

  # 請求書発行依頼の表示
  def print_request
    @invoice = SalesInvoice.find params[:id]
  end
  
  def destroy
    SalesInvoice.find(params[:id]).destroy
    redirect_to :action => 'list'
  end
end
